@@ -78,7 +78,7 @@ window_menu_help (BonoboUIComponent *uic,
 				NULL,
 				GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_ERROR,
-				GTK_BUTTONS_CLOSE,
+				GTK_BUTTONS_OK,
 				_("There was an error displaying help: %s"),
 				error->message);
 
