@@ -38,6 +38,70 @@ static GSList *registered_applets = NULL;
 static GSList *queued_position_saves = NULL;
 static guint   queued_position_source = 0;
 
+
+static void
+panel_applet_set_dnd_enabled (AppletInfo *info,
+			      gboolean    dnd_enabled)
+{
+	switch (info->type) {
+	case PANEL_OBJECT_DRAWER:
+		panel_drawer_set_dnd_enabled (info->data, dnd_enabled);
+		break;
+	case PANEL_OBJECT_MENU:
+		panel_menu_button_set_dnd_enabled (PANEL_MENU_BUTTON (info->widget),
+						   dnd_enabled);
+		break;
+	case PANEL_OBJECT_LAUNCHER:
+		panel_launcher_set_dnd_enabled (info->data, dnd_enabled);
+		break;
+	case PANEL_OBJECT_BONOBO:
+		break;
+	case PANEL_OBJECT_LOGOUT:
+	case PANEL_OBJECT_LOCK:
+	case PANEL_OBJECT_ACTION:
+		panel_action_button_set_dnd_enabled (PANEL_ACTION_BUTTON (info->widget),
+						     dnd_enabled);
+		break;
+	case PANEL_OBJECT_MENU_BAR:
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+
+}
+
+gboolean
+panel_applet_toggle_locked (AppletInfo *info)
+{
+	PanelWidget *panel_widget;
+	gboolean     locked;
+
+	panel_widget = PANEL_WIDGET (info->widget->parent);
+	
+	locked = panel_widget_toggle_applet_locked (panel_widget, info->widget);
+
+	panel_applet_save_position (info, info->id, TRUE);
+	panel_applet_set_dnd_enabled (info, !locked);
+
+	return locked;
+}
+
+static void
+panel_applet_lock (GtkWidget  *menuitem,
+		   AppletInfo *info)
+{
+	gboolean locked;
+
+	locked = panel_applet_toggle_locked (info);
+
+	gtk_label_set_label (GTK_LABEL (GTK_BIN (menuitem)->child),
+			     locked ? _("Un_lock") : _("_Lock"));
+
+	if (info->move_item)
+		gtk_widget_set_sensitive (info->move_item, !locked);
+}
+
 static void
 move_applet_callback (GtkWidget *widget, AppletInfo *info)
 {
@@ -63,14 +127,15 @@ panel_applet_clean (AppletInfo *info)
 {
 	g_return_if_fail (info != NULL);
 
-	if (info->type == PANEL_OBJECT_LAUNCHER) {
-		Launcher   *launcher = info->data;
-		const char *location;
-
-		location = gnome_desktop_item_get_location (launcher->ditem);
-
-		if (location)
-			 unlink (location);
+	switch (info->type) {
+	case PANEL_OBJECT_LAUNCHER:
+		panel_launcher_delete (info->data);
+		break;
+	case PANEL_OBJECT_DRAWER:
+		panel_drawer_delete (info->data);
+		break;
+	default:
+		break;
 	}
 
 	if (info->widget) {
@@ -372,7 +437,8 @@ add_to_submenus (AppletInfo *info,
 }
 
 static GtkWidget *
-panel_applet_create_menu (AppletInfo *info)
+panel_applet_create_menu (AppletInfo  *info,
+			  PanelWidget *panel_widget)
 {
 	GtkWidget *menu;
 	GtkWidget *menuitem;
@@ -398,34 +464,42 @@ panel_applet_create_menu (AppletInfo *info)
 
 	if (!commie_mode) {
 		GtkWidget *image;
+		gboolean   locked;
+
+		locked = panel_widget_get_applet_locked (panel_widget, info->widget);
 
 		menuitem = gtk_separator_menu_item_new ();
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 		gtk_widget_show (menuitem);
 
 		menuitem = gtk_image_menu_item_new ();
-
 		image = gtk_image_new_from_stock (GTK_STOCK_REMOVE,
 						  GTK_ICON_SIZE_MENU);
-
 		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image , _("_Remove From Panel"));
-
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (applet_remove_callback), info);
-
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 
 		menuitem = gtk_image_menu_item_new ();
-
-		/* FIXME: should have a "Move" pixmap.
-		 */
+		image = gtk_image_new ();
+		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image, locked ? _("Un_lock") : _("_Lock"));
+		g_signal_connect (menuitem, "activate",
+				  G_CALLBACK (panel_applet_lock), info);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+		
+		menuitem = gtk_image_menu_item_new ();
 		image = gtk_image_new ();
 		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image, _("_Move"));
-
+		gtk_widget_set_sensitive (menuitem, !locked);
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (move_applet_callback), info);
-
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+
+		g_assert (info->move_item == NULL);
+
+		info->move_item = menuitem;
+		g_object_add_weak_pointer (G_OBJECT (menuitem),
+					   (gpointer *) &info->move_item);
 	}
 
 	return menu;
@@ -465,7 +539,7 @@ applet_show_menu (AppletInfo     *info,
 	panel_widget = PANEL_WIDGET (info->widget->parent);
 
 	if (!info->menu)
-		info->menu = panel_applet_create_menu (info);
+		info->menu = panel_applet_create_menu (info, panel_widget);
 
 	g_assert (info->menu);
 
@@ -587,11 +661,12 @@ panel_applet_destroy (GtkWidget  *widget,
 }
 
 typedef struct {
-	PanelObjectType  type;
-	PanelWidget     *panel_widget;
-	int              position;
 	char            *id;
-	gulong           destroy_handler;
+	PanelObjectType  type;
+	char            *toplevel_id;
+	int              position;
+	guint            right_stick : 1;
+	guint            locked : 1;
 } PanelAppletToLoad;
 
 static GSList *panel_applets_to_load = NULL;
@@ -599,51 +674,74 @@ static GSList *panel_applets_to_load = NULL;
 static void
 free_applet_to_load (PanelAppletToLoad *applet)
 {
-	if (applet->destroy_handler)
-		g_signal_handler_disconnect (applet->panel_widget,
-					     applet->destroy_handler);
-	applet->destroy_handler = 0;
-
 	g_free (applet->id);
 	applet->id = NULL;
 
+	g_free (applet->toplevel_id);
+	applet->toplevel_id = NULL;
+
 	g_free (applet);
 }
 
 static gboolean
 panel_applet_load_idle_handler (gpointer dummy)
 {
-	PanelAppletToLoad *applet;
+	PanelAppletToLoad *applet = NULL;
+	PanelToplevel     *toplevel = NULL;
+	PanelWidget       *panel_widget;
+	GSList            *l;
 
 	if (!panel_applets_to_load)
 		return FALSE;
 
-	applet = (PanelAppletToLoad *) panel_applets_to_load->data;
-	panel_applets_to_load->data = NULL;
-	panel_applets_to_load =
-		g_slist_delete_link (panel_applets_to_load,
-				     panel_applets_to_load);
+	for (l = panel_applets_to_load; l; l = l->next) {
+		applet = l->data;
+
+		toplevel = panel_profile_get_toplevel_by_id (applet->toplevel_id);
+		if (toplevel)
+			break;
+	}
+
+	if (!l) {
+		/* All the rest of the applets don't have a panel */
+		for (l = panel_applets_to_load; l; l = l->next)
+			free_applet_to_load (l->data);
+		g_slist_free (panel_applets_to_load);
+		panel_applets_to_load = NULL;
+		return FALSE;
+	}
+
+	panel_applets_to_load = g_slist_delete_link (panel_applets_to_load, l);
+
+	panel_widget = panel_toplevel_get_panel_widget (toplevel);
+
+	if (applet->right_stick && !panel_widget->packed)
+		applet->position = panel_widget->size - applet->position;
 
 	switch (applet->type) {
 	case PANEL_OBJECT_BONOBO:
 		panel_applet_frame_load_from_gconf (
-					applet->panel_widget,
+					panel_widget,
+					applet->locked,
 					applet->position,
 					applet->id);
 		break;
 	case PANEL_OBJECT_DRAWER:
-		drawer_load_from_gconf (applet->panel_widget,
+		drawer_load_from_gconf (panel_widget,
+					applet->locked,
 					applet->position,
 					applet->id);
 		break;
 	case PANEL_OBJECT_MENU:
-		panel_menu_button_load_from_gconf (applet->panel_widget,
+		panel_menu_button_load_from_gconf (panel_widget,
+						   applet->locked,
 						   applet->position,
 						   TRUE,
 						   applet->id);
 		break;
 	case PANEL_OBJECT_LAUNCHER:
-		launcher_load_from_gconf (applet->panel_widget,
+		launcher_load_from_gconf (panel_widget,
+					  applet->locked,
 					  applet->position,
 					  applet->id);
 		break;
@@ -651,21 +749,24 @@ panel_applet_load_idle_handler (gpointer dummy)
 	case PANEL_OBJECT_LOCK:
 		panel_action_button_load_compatible (
 				applet->type,
-				applet->panel_widget,
+				panel_widget,
+				applet->locked,
 				applet->position,
 				TRUE,
 				applet->id);
 		break;
 	case PANEL_OBJECT_ACTION:
 		panel_action_button_load_from_gconf (
-				applet->panel_widget,
+				panel_widget,
+				applet->locked,
 				applet->position,
 				TRUE,
 				applet->id);
 		break;
 	case PANEL_OBJECT_MENU_BAR:
 		panel_menu_bar_load_from_gconf (
-				applet->panel_widget,
+				panel_widget,
+				applet->locked,
 				applet->position,
 				TRUE,
 				applet->id);
@@ -679,49 +780,29 @@ panel_applet_load_idle_handler (gpointer dummy)
 	return TRUE;
 }
 
-static void
-panel_destroyed_while_loading (GtkWidget *panel, gpointer data)
-{
-	PanelAppletToLoad *applet = data;
-
-	applet->destroy_handler = 0;
-
-	panel_applets_to_load =
-		g_slist_remove (panel_applets_to_load, applet);
-
-	free_applet_to_load (applet);
-}
-
 void
 panel_applet_queue_applet_to_load (char            *id,
 				   PanelObjectType  type,
-				   PanelToplevel   *toplevel,
+				   char            *toplevel_id,
 				   int              position,
-				   gboolean         right_stick)
+				   gboolean         right_stick,
+				   gboolean         locked)
 {
 	PanelAppletToLoad *applet;
-	PanelWidget       *panel_widget;
 
-	if (!toplevel) {
+	if (!toplevel_id) {
 		g_warning ("No toplevel on which to load object '%s'\n", id);
 		return;
 	}
 
-	panel_widget = panel_toplevel_get_panel_widget (toplevel);
-
-	if (right_stick && !panel_widget->packed)
-		position = panel_widget->size - position;
-
 	applet = g_new0 (PanelAppletToLoad, 1);
 
-	applet->type            = type;
-	applet->panel_widget    = panel_widget;
-	applet->position        = position;
-	applet->id              = id;
-	applet->destroy_handler =
-			g_signal_connect (panel_widget, "destroy",
-					  G_CALLBACK (panel_destroyed_while_loading),
-					  applet);
+	applet->id          = id;
+	applet->type        = type;
+	applet->toplevel_id = toplevel_id;
+	applet->position    = position;
+	applet->right_stick = right_stick != FALSE;
+	applet->locked      = locked != FALSE;
 
 	panel_applets_to_load = g_slist_prepend (panel_applets_to_load, applet);
 }
@@ -730,8 +811,10 @@ static int
 panel_applet_compare (const PanelAppletToLoad *a,
 		      const PanelAppletToLoad *b)
 {
-	if (a->panel_widget != b->panel_widget)
-		return a->panel_widget - b->panel_widget;
+	int c;
+
+	if ((c = strcmp (a->toplevel_id, b->toplevel_id)))
+		return c;
 	else
 		return a->position - b->position;
 }
@@ -789,6 +872,7 @@ panel_applet_save_position (AppletInfo *applet_info,
 {
 	PanelGConfKeyType  key_type;
 	GConfClient       *client;
+	PanelWidget       *panel_widget;
 	const char        *profile;
 	const char        *key;
 	const char        *toplevel_id;
@@ -818,6 +902,8 @@ panel_applet_save_position (AppletInfo *applet_info,
 	profile = panel_profile_get_name ();
 
 	key_type = applet_info->type == PANEL_OBJECT_BONOBO ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
+	
+	panel_widget = PANEL_WIDGET (applet_info->widget->parent);
 
 	key = panel_gconf_full_key (key_type, profile, id, "toplevel_id");
 	gconf_client_set_string (client, key, toplevel_id, NULL);
@@ -828,11 +914,16 @@ panel_applet_save_position (AppletInfo *applet_info,
 	gconf_client_set_bool (client, key, right_stick, NULL);
 
 	position = panel_applet_get_position (applet_info);
-	if (right_stick && !PANEL_WIDGET (applet_info->widget->parent)->packed)
-		position = PANEL_WIDGET (applet_info->widget->parent)->size - position;
+	if (right_stick && !panel_widget->packed)
+		position = panel_widget->size - position;
 
 	key = panel_gconf_full_key (key_type, profile, id, "position");
 	gconf_client_set_int (client, key, position, NULL);
+	
+	key = panel_gconf_full_key (key_type, profile, id, "locked");
+	gconf_client_set_bool (client, key, 
+			       panel_widget_get_applet_locked (panel_widget, applet_info->widget),
+			       NULL);
 }
 
 const char *
@@ -865,19 +956,34 @@ panel_applet_list_applets (void)
 	return registered_applets;
 }
 
+AppletInfo *
+panel_applet_get_by_type (PanelObjectType object_type)
+{
+	GSList *l;
+
+	for (l = registered_applets; l; l = l->next) {
+		AppletInfo *info = l->data;
+
+		if (info->type == object_type)
+			return info;
+	}
+
+	return NULL;
+}
+
 AppletInfo *
 panel_applet_register (GtkWidget       *applet,
 		       gpointer         data,
 		       GDestroyNotify   data_destroy,
 		       PanelWidget     *panel,
+		       gboolean         locked,
 		       gint             pos,
 		       gboolean         exactpos,
 		       PanelObjectType  type,
 		       const char      *id)
 {
 	AppletInfo *info;
-	int newpos;
-	gboolean insert_at_pos;
+	gboolean    insert_at_pos = FALSE;
 	
 	g_return_val_if_fail (applet != NULL && panel != NULL, NULL);
 
@@ -894,6 +1000,7 @@ panel_applet_register (GtkWidget       *applet,
 	info->data = data;
 	info->data_destroy = data_destroy;
 	info->user_menu = NULL;
+	info->move_item = NULL;
 
 	g_object_set_data (G_OBJECT (applet), "applet_info", info);
 
@@ -915,23 +1022,15 @@ panel_applet_register (GtkWidget       *applet,
 
 	registered_applets = g_slist_append (registered_applets, info);
 
-	/*add at the beginning if pos == -1*/
-	if (pos >= 0) {
-		newpos = pos;
-		insert_at_pos = FALSE;
-	} else {
-		newpos = 0;
-		insert_at_pos = FALSE;
-	}
 	/* if exact pos is on then insert at that precise location */
 	if (exactpos)
 		insert_at_pos = TRUE;
 
-	if (panel_widget_add (panel, applet, newpos, insert_at_pos) == -1) {
+	if (panel_widget_add (panel, applet, locked, pos, insert_at_pos) == -1) {
 		GSList *l;
 
 		for (l = panels; l; l = l->next)
-			if (panel_widget_add (panel, applet, 0, TRUE) != -1)
+			if (panel_widget_add (panel, applet, locked, 0, TRUE) != -1)
 				break;
 
 		if (!l) {
@@ -958,6 +1057,8 @@ panel_applet_register (GtkWidget       *applet,
 			  G_CALLBACK (panel_applet_destroy),
 			  info);
 
+	panel_applet_set_dnd_enabled (info, !locked);
+
 	gtk_widget_show_all (applet);
 
 	orientation_change (info, panel);
