@@ -39,7 +39,6 @@
 
 #include "button-widget.h"
 #include "distribution.h"
-#include "gnome-run.h"
 #include "launcher.h"
 #include "nothing.h"
 #include "menu-fentry.h"
@@ -65,6 +64,7 @@
 #include "panel-menu-button.h"
 #include "panel-globals.h"
 #include "panel-properties-dialog.h"
+#include "panel-run-dialog.h"
 
 #undef MENU_DEBUG
 
@@ -127,6 +127,16 @@ static AppletMenuInfo action_buttons_info [] = {
 	  PANEL_ACTION_LOCK,
 	  "ACTION:lock:NEW",
 	  TRUE },
+
+	{ N_("Force Quit"),
+	  N_("Actions"),
+	  "Actions",
+	  "Force a misbehaving application to quit",
+	  NULL,
+	  PANEL_STOCK_FORCE_QUIT,
+	  PANEL_ACTION_FORCE_QUIT,
+	  "ACTION:force-quit:NEW",
+	  TRUE },
 	  
 	{ N_("Screenshot"),
 	  N_("Actions"),
@@ -1177,7 +1187,7 @@ add_to_run_dialog (GtkWidget    *widget,
 				item, GNOME_DESKTOP_ITEM_URL);
 
 		if (exec != NULL)
-			show_run_dialog_with_text (menuitem_to_screen (sim->menuitem), exec);
+			panel_run_dialog_present_with_text (menuitem_to_screen (sim->menuitem), exec);
 		else
 			panel_error_dialog (
 				menuitem_to_screen (sim->menuitem),
@@ -1308,7 +1318,8 @@ add_drawers_from_dir (const char *dirname, const char *name,
 			continue;
 		}
 			
-		if (is_ext2 (mfile->name, ".desktop", ".kdelnk") &&
+		if ((g_str_has_suffix (mfile->name, ".desktop") ||
+		     g_str_has_suffix (mfile->name, ".kdelnk")) &&
 		    g_file_test (filename, G_FILE_TEST_EXISTS))
 			panel_launcher_create (newpanel->toplevel, G_MAXINT/2, filename);
 	}
@@ -3317,7 +3328,7 @@ create_root_menu (GtkWidget   *root_menu,
 		menuitem = gtk_image_menu_item_new ();
 		setup_stock_menu_item (
 			menuitem, panel_menu_icon_get_size (),
-			PANEL_STOCK_RUN, _("Run Application"));
+			PANEL_STOCK_RUN, _("Run Application..."));
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (panel_action_run_program),
 				  NULL);
